function reso=reliability(res,w,p,np)
 
// PURPOSE: estimate the reliability of the variables in a
// regression
// ------------------------------------------------------------
// INPUT:
// * res = the results of a -univariate- regression
// * w = a (1 x 4) vector of reliability outcome:
//   - w(1) = significant only on a sub-sample
//   - w(2) = significant only on the sub-samples
//   - w(3) = significant only on the whole sample
//   - w(4) = significant on the whole sample and on a
//            sub_sample
// * p = a size level
// * np = 'noprint' if the user does not want to print the
//   results
// ------------------------------------------------------------
// OUTPUT:
// reso = a tlist with:
// - res('meth')='reliability'
// - res('namey')= a string, the name of the endogenous
// variable in the regression
// - res('namex')= the string vector of the names of the
// exogenous variables in the regression
// - res('reliab')= the real vector of the reliability of the
// coefficients
// ------------------------------------------------------------
// Copyright: Eric Dubois 2006
 
[nargout,nargin]=argn(0)
 
if nargin < 4 then
   prt=%t
   if nargin <3 then
      p=0.05
   end
else
   if np ~= 'noprint' then
      warning('argin 4 in should be ''noprint''')
   end
   prt=%f
end
 
y=res('y')
x=res('x')
nvar=res('nvar')
meth=res('meth')
nobs=res('nobs')
halft=floor(nobs/2)
x1=x(1:halft,:)
x2=x(halft+1:nobs,:)
y1=y(1:halft,:)
y2=y(halft+1:nobs,:)
 
select meth
case 'ols' then
   meth='ols1'
 
case 'Newey-West''s HAC' then
   meth = 'nwest1'
 
case 'probit' then
   meth = 'probit2'
 
end

if or(svd(x1) < size(x1,2)*%eps) then
   reso='variables are colinear over the first half of the data'

elseif or(svd(x2) < size(x2,2)*%eps) then
   reso='variables are colinear over the second half of the data'

else   
   // retrieve the estimation results over each sub-sample
   execstr('r1='+meth+'(y1,x1)')
   execstr('r2='+meth+'(y2,x2)')
   reliab=zeros(nvar,1)
 
   for i=1:size(x,2)
      pall=bool2s(res('pvalue')(i) < p)
      p1=bool2s(r1('pvalue')(i) < p)
      p2=bool2s(r2('pvalue')(i) < p)
 
     // if pall, p1, p2 false then 0
     // if pall false and p1 or p2 true and the other false then w(1)
     // if pall false and p1 and p2 true then w(2)
     // if pall true and p1 and p2 false then w(3)
     // if pall true and one between p1 and p2 true then w(4)
     // if pall, p1, p2 true then 1
      reliab(i)=w(1)*(p1+p2)+...
               (w(2)-2*w(1))*p1*p2+...
                w(3)*pall+...
               (w(4)-w(3)-w(1))*pall*(p1*(1-p2)+p2*(1-p1))+...
               (1-w(4)-w(1))*pall*p1*p2
 
   end
 
   reso=tlist(['results';'meth';'namey';'namex';'reliab'],...
   'reliability',res('namey'),res('namex'),reliab)
 
   if prt then
      prt_reliab(reso)
   end
end
 
endfunction
