function [yhf,retal] = etalcalinsee(grocer_namey,varargin)

// PURPOSE: provide Insee's method for desaggegating time
// series with a high frequency indicator. The regression
// allows for:
//    (a) AR(1) or RW résiduals 
//    (b) inclusion or exclusion of a constant 
// ------------------------------------------------------------
// INPUT: 
// * grocer_namey  = low frequency series
// * varargin = strings that can be
//  .'s=xx' frequency of the desired output and, accordingly, 
//    of the indicators (default: 4)
//  .'alpha =xx' where w is significance level for the constant
//    default 0.1, that is 10%)
//  .'mod=''rw''' if the residuals are supposed to be a RW or
//   'mod=''ar''' if the residuals are supposed to be an AR(1)
//  (default: model with maximum likelihood is chosen)
//  .'bench=''xxx''' where xxx is the name of the series the
//    user wants to compare the result with (for instance to 
//    compare the method with another one) 
//  .'noprint' if the user does not want to print the results
// ------------------------------------------------------------
// OUTPUT: 
// * yhf = the disaggregated series
// * retal = a results tlist with: 
//   - retal('meth') = 'Insee''s disaggregation'
//   - retal('y annu') = annual series
//   - retal('y estim') = annual transformed series (that is
//   differentiated if residuals follow a RW, levle if not)
//   - retal('x annu') = a (N x 1) vector of annual data
//   - retal('x estim') = a (N x 1) vector of transformed annual
//   data
//   - retal('x trim') = a (n x k) matrix of exogenous high
//   frequency indicators
//   - retal('nobs') = # of observations in the regression
//   - retal('resid estim') = a (n x 1) vector of regression 
//   residuals
//   - retal('beta') = estimated coefficients of the annual 
//   regression
//   - retal('tstat') = t stats
//   - retal('pvalue') = pvalue of the betas
//   - retal('prescte') = boolean indicating the presence or
//   absence of a constant in the regression
//   - retal('llike') = the log-likelihood
//   - retal('rho') = autcorrelation coefficient of residuals: 
//  . if rho  = ]-1,1[ then the model is estimated in levels
//  . if rho = 1 then the model is estimated in differences
//   - retal('trend') = trend
//   . retal('trend') = 1 if retal('prescte') = %t and
//     rho = 1 
//   . retal('trend') = 2 if retal('prescte') = %t and 
//     rho ~= 1 
//   . retal('trend') = 0 in other cases  
//   - retal('prov y annu') = provisional value for y (and
//   therefore not used in estimation)
//   - retal('x annu aug') = (N x l) matrix of regressors,
//   including the constant or trend if necessary
//   - retal('x trim aug') = (n x l) matrix of regressors,
//   including the constant or trend if necessary
//   - retal('yhat annu') = (N x 1) adjusted annual y
//   - retal('resid annu') = (N x 1)annual residual
//   - retal('resid annu report') = annual residual extended
//   to the incomplete year with the estimated model
//   - retal('prov resid annu') = annual residual extended
//   using the last residual ("provisional year")
//   - retal('high freq yhat') = high frequency adjusted y
//   - retal('prov high freq yhat') = high frequency adjusted y
//   extended using the last residuals ("provisional years")
//   - retal('high freq resid') = high frequency residual in TS
//   form
//   - retal('prov high freq resid') = high frequency residual 
//   extended using the last residuals ("provisional years") in
//   TS form
//   - retal('prests') = boolean indicating the presence or
//     absence of a time series in the regression
//   - retal('namey') = name of the y variable
//   - retal('namex') = name of the x variables
//   ------------------------------------------------------------
// Copyright E. Michaux (2004)
// http://dubois.ensae.net/grocer.html

// set defaults
grocer_s = 4
grocer_alpha = 0.1
grocer_affich  = 1 
grocer_bench = 'no'
grocer_mod = 'no'
grocer_prt = %t
grocer_ny_prov = 1

grocer_nargin=length(varargin)
for grocer_i=grocer_nargin:-1:1
   if typeof(varargin(grocer_i)) == 'string' then
      str1=part(varargin(grocer_i),1:1) 
      str3=part(varargin(grocer_i),1:3) 
      str5=part(varargin(grocer_i),1:5) 
      if str1 == 's' then
         execstr('grocer_'+varargin(grocer_i))
         varargin(grocer_i)=null()
      elseif str3 == 'mod' then
         execstr('grocer_'+varargin(grocer_i))
         varargin(grocer_i)=null()
      elseif str5 == 'bench' | str5 == 'alpha' then
         execstr('grocer_'+varargin(grocer_i))
         varargin(grocer_i)=null()
      elseif part(varargin(grocer_i),1:7) == 'ny_prov'
         execstr('grocer_'+varargin(grocer_i))
         varargin(grocer_i)=null()
      elseif varargin(grocer_i) == 'noprint' then
         varargin(grocer_i)=null()
         grocer_prt=%f         
      end
   elseif typeof(varargin(grocer_i)) ~= 'constant' &...
   typeof(varargin(grocer_i)) ~= 'ts' then
      error('wrong type for entry number '+string(grocer_i+2))
   end
end

[y,x,namey,namexos,prests,boundsvarb,s,boundsy]=explo_agreg(-1,grocer_namey,varargin(:))

if s ~= 4 & s ~= 12 then 
 error("bad frequency: only monthly and quarterly indicators allowed")
end 

// provides the results from the regression of the vector y
// on the vector x
retal = etalcalinsee0(y,x,grocer_s,grocer_alpha,grocer_mod,grocer_ny_prov)

//// saves the names, the bounds if the regression involves ts
retal(1)($+1) = 'prests'
retal(1)($+1) = 'namex'
retal(1)($+1) = 'namey'

retal('prests')=prests
retal('namex') =namexos
retal('namey') =namey


// transform the results into TS
boxd = boundsvarb(1)
boxf = boundsvarb(2)

yhf=reshape(retal('prov high freq resid'),boxd) 
retal('high freq yhat') = reshape(retal('high freq yhat'),boxd) 
retal('prov high freq yhat') = reshape(retal('prov high freq yhat'),boxd) 
retal('high freq resid') = reshape(retal('high freq resid'),boxd) 
retal('prov high freq resid') = yhf

boyd =  boundsy(1)
retal('yhat annu') = reshape(retal('yhat annu'),boyd) 
retal('resid annu') = reshape(retal('resid annu'),boyd)

if grocer_prt then
   prtetalcal(retal)      

   if grocer_bench ~= 'no'

     yhat = retal('high freq yhat')    
     yhat_prov = retal('prov high freq yhat')    
   
     ng=winsid()  // recover the list of open graphic w
     ng = ng(size(ng,2)) +1   // add 1 to the last open window 

     pltseries(grocer_bench,'yhat_prov','styleg=2',...
     'title = disaggregation of '+retal('namey')+'','bounds=['''+boxd+''' '''+boxf+''']',...
     'legend=[observed;disaggregated]','window=ng','styleg=6')
 
   end 
end
 
endfunction
