function retal = etalcalinsee0(ly,hx,s,alpha,mod,ny_prov)

// PURPOSE: provide Insee's method for desaggegating time
// series with a high frequency indicator. The regression
// allows for:
//    (a) AR(1) or RW résiduals 
//    (b) inclusion or exclusion of a constant 
// ------------------------------------------------------------
// INPUT: 
// * ly  = donnees de compte annuel à partir de laquelle on étalonne (vecteur)
// * hx = indicateur servant pour l'etalonnage (vecteur) 
// * s = fréquence de raélisation de l'étalonnage-calage (12 = mensuel, 4 = trim)
// * alpha = seuil significativité du test sur la constante
// ------------------------------------------------------------
// OUTPUT: 
// * retal = a results tlist with: 
//   - retal('meth') = 'Insee''s disaggregation'
//   - retal('y annu') = annual series
//   - retal('y estim') = annual transformed series (that is
//   differentiated if residuals follow a RW, levle if not)
//   - retal('x annu') = a (N x 1) vector of annual data
//   - retal('x estim') = a (N x 1) vector of transformed annual
//   data
//   - retal('x trim') = a (n x k) matrix of exogenous high
//   frequency indicators
//   - retal('nobs') = # of observations in the regression
//   - retal('resid estim') = a (n x 1) vector of regression 
//   residuals
//   - retal('beta') = estimated coefficients of the annual 
//   regression
//   - retal('tstat') = t stats
//   - retal('pvalue') = pvalue of the betas
//   - retal('prescte') = boolean indicating the presence or
//   absence of a constant in the regression
//   - retal('llike') = the log-likelihood
//   - retal('rho') = autcorrelation coefficient of residuals: 
//  . if rho  = ]-1,1[ then the model is estimated in levels
//  . if rho = 1 then the model is estimated in differences
//   - retal('trend') = trend
//   . retal('trend') = 1 if retal('prescte') = %t and
//     rho = 1 
//   . retal('trend') = 2 if retal('prescte') = %t and 
//     rho ~= 1 
//   . retal('trend') = 0 in other cases  
//   - retal('prov y annu') = provisional value for y (and
//   therefore not used in estimation)
//   - retal('x annu aug') = (N x l) matrix of regressors,
//   including the constant or trend if necessary
//   - retal('x trim aug') = (n x l) matrix of regressors,
//   including the constant or trend if necessary
//   - retal('yhat annu') = (N x 1) adjusted annual y
//   - retal('resid annu') = (N x 1)annual residual
//   - retal('resid annu report') = annual residual extended
//   to the incomplete year with the estimated model
//   - retal('prov resid annu') = annual residual extended
//   using the last residual ("provisional year")
//   - retal('high freq yhat') = high frequency adjusted y
//   - retal('prov high freq yhat') = high frequency adjusted y
//   extended using the last residuals ("provisional years")
//   - retal('high freq resid') = high frequency residual 
//   - retal('prov high freq resid') = high frequency residual 
//   extended using the last residuals ("provisional years")
//   - retal('prests') = boolean indicating the presence or
//     absence of a time series in the regression
//   - retal('namey') = name of the y variable
//   - retal('namex') = name of the x variables
//   ------------------------------------------------------------
// Copyright E. Michaux (2004)
// http://dubois.ensae.net/grocer.html


// remove last pints from the estimation
ly_end = ly(size(ly,1)-ny_prov+1:size(ly,1)) ; 
ly = ly(1:size(ly,1)-ny_prov) ;

// dimensions of the problem 
n = size(ly,1) ;  // n= nbd de lignes de ly ie nb années
[N p] = size(hx) ; // N nb lignes de hx donc nb de trim ou mois et p nb de colonnes donc d'indic  

// build the aggregation matrix 
[c,pred] = matagreg(n,N,s)  ; 
lx = c*hx ;

// find the best annual model between the annual data
// and the high frequency indicator
retal =selecmodel(ly,lx,s,alpha,mod)
retal(1)($+1) = 'x trim'
retal(1)($+1) = 'annu prov y'
retal(1)($+1) ='high freq'

retal('x trim') = hx 
retal('annu prov y') = ly_end
retal('high freq') = s

// recalcul de x selon le modèle sélectionné 
retal = agregx(retal,c) ;

// estimation de la production annuelle est des résidus 
retal = foreannu(retal) 
retal = lissquadra(retal,'pas') ;

// estimation de la  production trimestrielle 
retal = foreresid(retal,pred)
retal = foretrim(retal,pred)

endfunction
