function [rridge]=ridge(grocer_namey0,varargin)
 
// PURPOSE: computes Hoerl-Kennard Ridge Regression
// ------------------------------------------------------------
// REFERENCES: David Birkes, Yadolah Dodge, 1993, "Alternative
// Methods of Regression" and Hoerl, Kennard, Baldwin, 1975
// "Ridge Regression: Some Simulations', Communcations in
// Statistics
// ------------------------------------------------------------
// INPUT:
// * grocer_namey0 = a time series, a real (nx1) vector or a
// string equal to the name of a time series or a (nx1) real
// vector between quotes
// * varargin = arguments which can be:
//   . a time series
//   . a real (nxp) vector
//   . a string equal to the name of a time series or a (nxp)
//     real vector between quotes
//   . the string 'noprint' if the user doesn't want to print
//     the results of the regression
//   . the string 'theta=xx' if the user wants to enter theta's
//     value (default is the one recommended by Hoerl and
//     Kennard)
//   . the string 'dropna' if the user wants to use in the
//     regression all dates with no NA value in any variable
//     (the main use of this option should be when dealing with
//     daily ts)
// ------------------------------------------------------------
// OUTPUT:
// rridge = a tlist with
//   . rridge('meth')  = 'ridge'
//   . rridge('y')     = y data vector
//   . rridge('x')     = x data matrix
//   . rridge('nobs')  = nobs
//   . rridge('nvar')  = nvars
//   . rridge('beta')  = bhat
//   . rridge('yhat')  = yhat
//   . rridge('resid') = residuals
//   . rridge('vcovar') = estimated variance-covariance matrix of
//     beta
//   . rridge('sige')  = estimated variance of the residuals
//   . rridge('sige')  = estimated variance of the residuals
//   . rridge('ser')  = standard error of the regression
//   . rridge('tstat') = t-stats
//   . rridge('pvalue') = pvalue of the betas
//   . rridge('dw')    = Durbin-Watson Statistic
//   . rridge('prescte') = boolean indicating the presence or
//     absence of a constant in the regression
//   . rridge('theta') = the scale factor theta
//   . rridge('rsqr')  = rsquared
//   . rridge('rbar')  = rbar-squared
//   . rridge('f')    = F-stat for the nullity of coefficients
//     other than the constant
//   . rridge('pvaluef') = its significance level
//   . rridge('prests') = boolean indicating the presence or
//     absence of a time series in the regression
//   . rridge('namey') = name of the y variable
//   . rridge('namex') = name of the x variables
//   . rridge('dropna') = boolean indicating if NAs had
//		   been droped
//   . rridge('bounds') = if there is a timeseries in the
//     regression, the bounds of the regression
//   . rridge('nonna') = vector indicating position of non-NAs
// ------------------------------------------------------------
// PRINTS: If the user has not given the argument 'noprint',
// the results of the regression and various diagnostics
// ------------------------------------------------------------
// NOTES:
// see also: rtrace
// ------------------------------------------------------------
// Copyright: Eric Dubois 2002-2007
// http://dubois.ensae.net/grocer.html
 
// adapted from:
// James P. LeSage, Dept of Economics
// University of Toledo
// 2801 W. Bancroft St,
// Toledo, OH 43606
// jpl@jpl.econ.utoledo.edu
 
grocer_prt=%t
grocer_dropna=%f
grocer_nargin=length(varargin)
 
// separate from the list of variable arguments the list of
// exogenous variables and other arguments
for grocer_i=grocer_nargin:-1:1
   if typeof(varargin(grocer_i)) == 'string' then
      if varargin(grocer_i) == 'noprint' then
         grocer_prt=%f
         varargin(grocer_i) = null()
      elseif varargin(grocer_i) == 'dropna' then
         grocer_dropna=%t
         varargin(grocer_i)=null()
      end
 
      if part(varargin(grocer_i),1:5) == 'theta'  then
         execstr('grocer_'+varargin(grocer_i))
         varargin(grocer_i) = null()
      end
   end
end
 
grocer_lx=varargin
[grocer_y,grocer_namey,grocer_x,grocer_namexos,grocer_prests,grocer_boundsvarb,nonna]=...
      explouniv(grocer_namey0,grocer_lx,[],['endogenous';'exogenous'],%t,grocer_dropna)
 
nobs2=size(grocer_y,1)
[nobs,nvar] = size(grocer_x);
if nobs~=nobs2 then
  error('x and y must have same # obs in ridge');
end
if nvar >= nobs then
   error('too many exogenous variables')
end
 
ridi = eye(nvar,nvar)
if ~exists('grocer_theta','local') then
// initial values if the user has not provided one for theta
   dfs = nobs - nvar - 1;
   b=ols0(grocer_y,grocer_x)
   ridi = diag(diag(grocer_x'*grocer_x));
   dif = grocer_x*b(:,1)-grocer_y;
   ssqerr = dif'*dif;
   grocer_theta = nvar*(ssqerr/dfs)/sum((ridi.^0.5*b).^2);
end
 
// transformation of the names into variables and verification of the existence of a constant
// variable
 
xpxi = inv(grocer_x'*grocer_x+ridi*grocer_theta);
bet = xpxi*(grocer_x'*grocer_y);
 
yhat = grocer_x*bet
resid = grocer_y-yhat
sigu =resid'*resid
sige= sigu/(nobs-nvar)
ser=sqrt(sige)
vcovar=sige*xpxi
tmp = diag(vcovar);
tstat = bet ./ sqrt(tmp)
 
ym = grocer_y-mean0(grocer_y);
ediff = resid(2:nobs)-resid(1:nobs-1)
dw = ediff'*ediff/sigu
// durbin-watson
 
df=nobs-nvar
pvalue=ones(nvar,1)
for i=1:nvar
   pvalue(i)=(1-cdft("PQ",abs(tstat(i)),df))*2
end
 
condindex=bkwols(grocer_x)
 
prescte=%f
if nvar ~= 1 then
   i=1
   while (i <= nvar) & ~prescte then
      // if all values are equal to the first one then,
      // the variable is constant
      prescte=and(grocer_x(:,i) == grocer_x(1,i))
      i=i+1
   end
end
 
rridge = tlist(['results';'meth';'y';'x';'nobs';'nvar';'beta';'yhat';...
'resid';'vcovar';'sige';'sigu';'ser';'tstat';'pvalue';'dw';'condindex';...
'prescte';'theta']...
,'ridge',grocer_y,grocer_x,nobs,nvar,bet,yhat,resid,vcovar,sige,sigu,ser,tstat,...
pvalue,dw,condindex,prescte,grocer_theta)
 
 
if prescte then
// there is a constant and at least another exogenous variable:
// R² and Rbar² make sense
  rsqr1 = sigu;
  rsqr2 = ym'*ym;
  // r-squared
  rridge(1)($+1)='rsqr'
  rsqr=1-rsqr1/rsqr2
  rridge('rsqr') =rsqr
  nobsm1=nobs-1
  nvarm1=nvar-1
  rridge(1)($+1)='rbar'
  rridge('rbar') = 1-rsqr1/df/rsqr2*nobsm1
  // rbar-squared
  f=rsqr/(1-rsqr)*df/(nvar-1)
  pvaluef=1-cdff("PQ",f,nvarm1,nobs-nvar)
  rridge(1)($+1)='f'
  rridge('f') = f
  rridge(1)($+1)='pvaluef'
  rridge('pvaluef') = pvaluef
end
 
// saves the names, the bounds if the regression involves ts
rridge(1)($+1) = 'prests'
rridge(1)($+1) = 'namex'
rridge(1)($+1) = 'namey'
rridge(1)($+1) = 'dropna'
rridge('prests')=grocer_prests
rridge('namex')=grocer_namexos
rridge('namey')=grocer_namey
rridge('dropna')=grocer_dropna
if grocer_prests then
   rridge(1)($+1) = 'bounds'
   rridge('bounds')=grocer_boundsvarb
end
if grocer_dropna then
   rridge(1)($+1)='nonna'
   rridge('nonna')=nonna
end
 
if grocer_prt then
// the user has not entered 'noprint' as an argument
// a command to activate if foutput has been previously definded:
// prtuniv(rridge,foutput)
   prt_ols(rridge,%io(2))
   pltuniv(rridge,'all')
end
endfunction
