function [irf_low,irf_upp]=irf_asy(results,mres,P,IRF,PHI,S,siz)
 
// PURPOSE: Calculates standard error of Impulse Response
// Function for VAR by means of the asymptotic formula
//-------------------------------------------------------------
// REFERENCES:
// * Hamilton, Time Series Analysis (1994)
// * Lütkepohl, Introduction to Time Series Analysis (1994)
// ------------------------------------------------------------
// INPUT:
// * results = results tlist returned by VAR
// * mres = decomposition method
// * P = matrix such that P*e = u
//       where u is the residual from the VAR regression
//             e is the residual to be shocked
// * IRF = ((S+1) x T) impulse response functions
// * PHI = (N*p x T) matrix of coefficients
// * S = # of periods
// * siz = size of the confidence band
//-------------------------------------------------------------
// OUTPUT:
// * irf_low = ((S+1) x T) lower range of impulse response
//                     confidence band
// * irf_upp = ((S+1) x T) upper range of impulse response
//                     confidence band
//-------------------------------------------------------------
// Copyright Eric Dubois 2002-2013
// http://dubois.ensae.net/grocer.html
 
N = results('neqs');
p = results('nlag');
nx=results('nx')
nobs = results('nobs');
sigma =results('sigma')
sig_vecA= results('vcovar')
index=1:N*p
for i=1:N-1
    index=[index [1:N*p]+i*(N*p+nx)]
end
sig_vecA=sig_vecA(index,index)
mult=cdfnor("X",0,1,1-siz/2,siz/2)
 
A=[zeros(N,N*p) ;eye(N*(p-1),N*p)]
v=zeros(N*(S+1),N)
j=eye(N,N*p)
dup=duplication(N)
dupp=inv(dup'*dup)*dup'
com=commutation(N)
eli=elimination(N)
 
for i=1:p
   A(1:N,1+(i-1)*N:i*N) = PHI(:,1+(p-i)*N:(p-i+1)*N)
end
 
select mres
 
case 'chol1'
   // retrieve the impulse response function for the original
   // residuals
   PHI=irf0(results('beta'),S,N,p,eye(N,N))
   sig_sig=2*dupp*(sigma.*.sigma)*dupp'/nobs
   h=eli'*inv(eli*((eye(N,N).*.P)*com + P.*.eye(N,N))*eli')
   v(1:N,:)=sqrt(matrix(diag(h*sig_sig*h'),N,N))
   for s=1:S
      G=(j*A'^(s-1)).*.PHI(1:N,:)
      for m=1:s-1
         G=G+(j*A'^(s-1-m).*.PHI(1+N*m:(m+1)*N,:))
      end
      cs=(P'.*.eye(N,N))*G
      csb=(eye(N,N).*.PHI(1+N*s:(s+1)*N,:))*h
      v(1+s*N:(s+1)*N,:)=sqrt(matrix(diag(cs*sig_vecA*cs'+...
                         csb*sig_sig*csb'),N,N))
   end
 
case 'chol2'
   // retrieve the impulse response function for the original
   // residuals
   PHI=irf0(results('beta'),S,N,p,eye(N,N))
   sig_sig=2*dupp*(sigma.*.sigma)*dupp'/nobs
   maux=zeros(N^2,N^2)
   for i=1:N
      maux((i-1)*N+i,(i-1)*N+i) = 1/sigma(i,i)
   end
   h=(diag(sqrt(diag(sigma))).*.eye(N,N))*eli'*...
     inv(eli*((eye(N,N).*.P)*com + P.*.eye(N,N))*eli')-...
     maux*(eye(N,N).*.P)*dup
   v(1:N,:)=sqrt(matrix(diag(h*sig_sig*h'),N,N))
   for s=1:S
      G=(j*A'^(s-1)).*.PHI(1:N,:)
      for m=1:s-1
         G=G+(j*A'^(s-1-m).*.PHI(1+N*m:(m+1)*N,:))
      end
      cs=(P'.*.eye(N,N))*G
      csb=(eye(N,N).*.PHI(1+N*s:(s+1)*N,:))*h
      v(1+s*N:(s+1)*N,:)=sqrt(matrix(diag(cs*sig_vecA*cs'+...
                         csb*sig_sig*csb'),N,N))
   end
 
case 'original'
   for s=1:S
      G=(j*A'^(s-1)).*.IRF(1:N,:)
      for m=1:s-1
         G=G+(j*A'^(s-1-m).*.IRF(1+N*m:(m+1)*N,:))
      end
      v(1+s*N:(s+1)*N,:)=sqrt(matrix(diag(G*sig_vecA*G'),N,N))
   end
 
end
 
irf_low=IRF-mult*v
irf_upp=IRF+mult*v
endfunction
