function [irf_low,irf_upp]=irf_mc1(res,mres,S,N,p,niter,siz)
 
// PURPOSE: Calculates standard error of Impulse Response
// Function for VAR by means of Monte-Carlo simulations on
// the coefficient matrix
// ------------------------------------------------------------
// INPUT:
// * res = results tlist returned by VAR
// * mres = decomposition method
// * N = # of endogenous variables in the VAR
// * S = # of periods
// * p = # of lags in the VAR
// * niter = # of draws in the simulation
// * siz = size of the confidence band
//-------------------------------------------------------------
// OUTPUT:
// * irf_low = ((S+1) x T) lower range of impulse response
//                     confidence band
// * irf_upp = ((S+1) x T) upper range of impulse response
//                     confidence band
//-------------------------------------------------------------
// Copyright Eric Dubois 2002-2013
// http://dubois.ensae.net/grocer.html
 
//============================================================
//    SET UP PARMS
//============================================================
 
nobs=res('nobs')
nx=res('nx')
bet=res('beta')(1:N*p,:)
sigma=res('sigma')
sig_vecA= res('vcovar')
index=1:N*p
for i=1:N-1
    index=[index [1:N*p]+i*(N*p+nx)]
end
sig_vecA=sig_vecA(index,index)

dup=duplication(N)
dupp=inv(dup'*dup)*dup'
sqrtsig_sig=(2*dupp*(sigma.*.sigma)*dupp'/nobs)^0.5
 
sIRF0=zeros(niter,(S+1)*N,N)
sIRF=zeros(niter,(S+1)*N*N)
irf_low=ones(N*(S+1),N)
irf_upp=ones(N*(S+1),N)

select res('meth')

case 'var' then
   drawN=grand(niter,"mn",bet(:),sig_vecA)

case 'restricted var' then
   vecbet=bet(:)
   indxf=find(diag(sig_vecA) ~= 0)
   drawN=zeros(N*p,niter) 
   drawN(indxf,:)=grand(niter,"mn",vecbet(indxf),sig_vecA(indxf,indxf))
end
 
i=0
for i =1:niter
// draw the betas
   sbeta=matrix(drawN(:,i),N*p,N)
 
   select mres
   case 'chol1'
// draw the residual matrix because of the Choleski
// decomposition
      v=matrix(dup*sqrtsig_sig*grand(N*(N+1)/2,1,'nor',0,1),N,N)
      ssigma=sigma+v
      if and(spec(ssigma) > 0) then
      //ssigma is definite positive
         P=chol(ssigma)'
         m=irf0(sbeta,S,N,p,P)
         sIRF(i,:)=matrix(m,-1,1)'
      else
         i=i-1
      end
 
   case 'chol2'
// draw the residual matrix because of the Choleski
// decomposition
      v=matrix(dup*sqrtsig_sig*grand(N*(N+1)/2,1,'nor',0,1),N,N)
      ssigma=sigma+v
      if and(real(spec(ssigma)) > 0) then
         //ssigma is definite positive
         Dsr = diag(sqrt(diag(ssigma)));
         P = chol(ssigma)'*inv(Dsr);
         m=irf0(sbeta,S,N,p,P)
         sIRF(i,:)=matrix(m,-1,1)'
      else
         i=i-1
      end
 
   case 'original'
      P=eye(N,N)
      m=irf0(sbeta,S,N,p,P)
      sIRF(i,:)=matrix(m,-1,1)'
   end
 
end
 
for j=0:S
   for k=1:N
      for l=1:N
         irf_sort=gsort(real(sIRF(:,N*j+l+(k-1)*N*(S+1))),'g','d')
         irf_low(N*j+l,k)=irf_sort(floor((1-siz)*niter))
         irf_upp(N*j+l,k)=irf_sort(floor(siz*niter))
      end
   end
end
endfunction
