function [grocer_mats,grocer_names,grocer_prests,grocer_b,grocer_nonna]=explon(grocer_vars,grocer_named,grocer_b,grocer_testna,grocer_dropna,grocer_mindat,grocer_lags)
 
// PURPOSE: explode n lists of variables into matrices, names
// and define corresponding bounds
// ------------------------------------------------------------
// INPUT:
// * grocer_vars = list of n:
//   - lists of variables
//   - matrix of names of variables
//   the variables being in the form of
//   * a time series
//   * a real vector,
//   * a real matrix or a string (the name of a variable with
//   one of the types cited above, between quotes)
//   * a matrix of strings, each one being the name of a
//   variable
//   * the string 'cte' or 'const' if the user wants a constant
//     to be included automatically
// * grocer_named = a (n x 1) or (1 x n) vector of strings
//   representing the name of variables not entered between
//   quotes
// * grocer_b = a (p x 1) string vector (of dates) (optional:
//   if not given the function either takes the existing bounds
//   or determines the bounds suitable to the given series)
// * grocer_testna = a boolean indicating whether the program
//   will test the existence of na’s values in the matrices of
//   values grocer_y and grocer_x
// * grocer_dropna = a boolean indicating whether the program
//   should keep only lines of matrices grocer_y and grocer_x
//   with non na values in both matrices (suppose that
//   grocer_testna has been set to %f)
// * grocer_lags = a (n x 1) vector, each coordinate indicating
//   the lag needed for the corresponding elements in the list
//   grocer_vars
// ------------------------------------------------------------
// OUTPUT:
// * grocer_mats = a list of n (T x ki) matrices
// * grocer_names = a list of n (ki x 1) matrices of names
// * grocer_prests = a boolean indicating whether there is a ts
// * grocer_b = a (2*p x 1) string matrix (of dates) or []
// * grocer_nonna = a (T x 1) vector of the indexes with non na values
// ------------------------------------------------------------
// Copyright: Eric Dubois 2006-2015
// http://dubois.ensae.net/grocer.html
 
[grocer_nargout,grocer_nargin] = argn(0)
grocer_n=length(grocer_vars) // number of different types of econometric objects
if grocer_nargin < 7 then
   grocer_lags=zeros(grocer_n,1)
end
if grocer_nargin < 6 then
   grocer_mindat=%f
end
 
if grocer_nargin < 5 then
   grocer_dropna=%f
end
 
if grocer_nargin < 4 then
   grocer_testna=%t
end
 
if ~exists('grocer_b','local') then
   // bounds have been entered into the function; they prevail over other existing bounds
   grocer_b=[]
end
 
if isempty(grocer_b) & exists('grocer_boundsvar') then
// bounds have been defined outside the function
   grocer_b=grocer_boundsvar
end

grocer_mats=list() // the list of grocer_n matrices to be used in the regession
grocer_names=list() // the list of grocer_n vectors of the corresponding variables names
grocer_listts=list() // the list of grocer_n ts for each type of econometric objects
grocer_listtsmat=list() // the list of grocer_n tsmat for each type of econometric objects
grocer_vec0=list() // the list of grocer_n real matrices for each type of econometric objects
grocer_indts0=list() // the list of grocer_n ts indexes for each type of econometric objects
grocer_indtsmat0=list() // the list of grocer_n tsmat indexes for each type of econometric objects
grocer_indvec0=list() // the list of grocer_n real matrices indexes for each type of econometric objects
grocer_indcte0=list() // the list of grocer_n real indexes of specific variables for each type of econometric objects
grocer_n0=list() // the list collecting the number of final variables for each type of econometric objects
grocer_prests=%f
grocer_isvec=%f
grocer_b0=grocer_b
 
for grocer_i=1:grocer_n
 
   grocer_li=grocer_vars(grocer_i)
 
// explode the list grocer_i into its various components (real matrices, ts, tsmat, specific variables)
// and their indexes
   [namei,listtsi,grocer_veci,indtsi,indveci,indctei,ni,listtsmati,indtsmati]=...
   explovars(grocer_li,grocer_named(grocer_i))
 
// add the components and indexes to the corresponding list
   grocer_names(grocer_i)=namei
   grocer_listts(grocer_i)=listtsi
   grocer_listtsmat(grocer_i)=listtsmati
   grocer_vec0(grocer_i)=grocer_veci
   grocer_indts0(grocer_i)=indtsi
   grocer_indtsmat0(grocer_i)=indtsmati
   grocer_indvec0(grocer_i)=indveci
   grocer_indcte0(grocer_i)=indctei
   grocer_n0(grocer_i)=ni
   if ~exists('ts1','local') & ~isempty(indtsi) then
      ts1=listtsi(1)
      fq=ts1('freq')
   elseif ~exists('tsmat1','local') & ~isempty(indtsmati) then
      tsmat1=listtsmati(1)
      fq=tsmat1('freq')
   end
   grocer_prests = grocer_prests | length(listtsi) ~= 0 | length(listtsmati) ~= 0
   grocer_isvec = grocer_isvec | length(grocer_veci) ~= 0
 
end
 
if grocer_prests then
   if grocer_isvec
      warning('your regression contains ts as well as vectors')
   end
   
   if grocer_b == [] then
      dates_start=[]
      dates_end=[]
      for i=1:grocer_n
         if ~isempty(grocer_listts(i)) then
            [datei_start,datei_end]=ts_span(grocer_listts(i))
            dates_start=[dates_start ; datei_start+grocer_lags(i)]
            dates_end=[dates_end ; datei_end]
         end
         if ~isempty(grocer_listtsmat(i)) then
            [datei_start,datei_end]=ts_span(grocer_listtsmat(i))
            dates_start=[dates_start ; datei_start+grocer_lags(i)]
            dates_end=[dates_end ; datei_end]
         end
      end
      if ~grocer_testna & grocer_mindat then
         dmin=min(dates_start)
         dmax=max(dates_end)
      else
         dmin=max(dates_start)
         dmax=min(dates_end)         
      end
      boundsnum=[dmin:dmax]'
      grocer_b=[num2date(dmin,fq) ; num2date(dmax,fq)]

   else
      boundsnum=[date2num(grocer_b(1)):date2num(grocer_b(2))]'
      for i=2:size(grocer_b,1)/2
         boundsnum=[boundsnum ; [date2num(grocer_b(2*i-1)):date2num(grocer_b(2*i))]']
      end

   end
   nobs=size(boundsnum,1) 
 
   for j=1:grocer_n
      lagj=grocer_lags(j)
      yj=%nan*ones(nobs+lagj,grocer_n0(j)) // grocer_yj = vector of values to fill
      bj=[boundsnum(1)-[lagj:-1:0]' ;boundsnum(2:$)]
      listtsj=grocer_listts(j)
      indtsj=grocer_indts0(j)
   // treat the j-th type of econometric object
      for k=1:size(indtsj,1)
         tsk=listtsj(k)
         tsk_dates=tsk('dates')
         start_yj=max(1,tsk_dates(1)-bj(1)+1)
         start_tsk=max(1,bj(1)-tsk_dates(1)+1)
         end_yj=nobs+lagj-max(0,bj($)-tsk_dates($))
         end_tsk=size(tsk_dates,1)-max(0,tsk_dates($)-bj($))
         yj(start_yj:end_yj,indtsj(k))=tsk('series')(start_tsk:end_tsk) // fill the values stemming from ts
      end

      listtsmatj=grocer_listtsmat(j)
      indtsmatj=grocer_indtsmat0(j)
      ind0=0
      for k=1:length(listtsmatj)
         tsmatk=listtsmatj(k)
         tsmatk_dates=tsmatk('dates')
         start_yj=max(1,tsmatk_dates(1)-bj(1)+1)
         start_tsk=max(1,bj(1)-tsmatk_dates(1)+1)
         end_yj=nobs+lagj-max(0,bj($)-tsmatk_dates($))
         end_tsk=size(tsmatk_dates,1)-max(0,tsmatk_dates($)-bj($))
         ind1=size(tsmatk('series'),2)
         yj(start_yj:end_yj,indtsmatj(ind0+[1:ind1]))=tsmatk('series')(start_tsk:end_tsk,:) // fill the values stemming from ts
         ind0=ind1
      end

      nmatj=size(grocer_vec0(j),1) // number of observations stemming from the real matrices
      if nmatj > 0 & nmatj ~= nobs then
         error('nobs of matrices do not match with those of ts')
      end
      yj(:,grocer_indvec0(j))=grocer_vec0(j) // fill the values stemming from real matrices
 
      indctei=grocer_indcte0(j) // index of the specific variables
      namei=grocer_names(j)
      for i=1:size(indctei,1)
         k=indctei(i)
         yj=deal_varspec(yj,namei(k),k,bj) // fill the matrix of values for the specific variables
      end
 
      grocer_mats(j)=yj
   end
 
 
else
// there is no ts: only real matrices and specifc objects
   fill=[] // indexes of econometric objects that contain explicit values
   for j=1:grocer_n
      if grocer_indvec0(j) ~= [] then
         fill=[fill j]
      end
   end
   empt=1:grocer_n // indexes of econometric objects that do not contain explicit values
   empt(fill)=[]
   nobs=size(grocer_vec0(fill(1)),1)-grocer_lags(1)
 
   for i=1:size(fill,2)
      j=fill(i)
      yj=ones(nobs+grocer_lags(j),grocer_n0(j))
      yj(:,grocer_indvec0(j))=grocer_vec0(j) // fill the concerned yj columns with their values
 
      indctej=grocer_indcte0(j)
      namej=grocer_names(j)
      for m=1:size(indctej,1)
         k=indctej(m)
         yj=deal_varspec(yj,namej(k),k,1:nobs+grocer_lags(j)) // fill specific variables
      end
 
      indcte=search_cte(yj) // indexes of constant variables
      if size(indcte,2) > 1 & j > 1 then
         warning('you have more than one constant in your '+grocer_named(j)+' set of variables')
      end
      grocer_mats(j)=yj
   end
 
   for i=1:size(empt,2)
      j=empt(i)
      indctej=grocer_indcte0(j)
      yj=ones(nobs+grocer_lags(j),grocer_n0(j))
      namej=grocer_names(j)
      for m=1:size(indctej,1)
         k=indctej(m)
         yj=deal_varspec(yj,namej(k),k,1:nobs+grocer_lags(j)) // fill specific variables
      end
 
      indcte=search_cte(yj) // indexes of constant variables

      if size(indcte,2) > 1 & j>1 then
         warning('you have more than one constant in your '+grocer_named(j)+' set of variables')
      end
      grocer_mats(j)=yj
   end
   grocer_b=[]
 
end
 
nonna=%t
mat=[]
for i=1:grocer_n
   mat_aux=[grocer_mats(i) mlag(grocer_mats(i),grocer_lags(i))]
   mat=[mat mat_aux(1+grocer_lags(i):$,:)]
end
 
ind_na=find(or(isnan(mat),'c'))
grocer_nonna=find(and(~isnan(mat),'c'))

if grocer_dropna then
   for j=1:grocer_n
      matj=grocer_mats(j)
      ind_nalags=ind_na+grocer_nlags(j)
      for i=1:grocer_nlags(j)-1
         ind_nalags=[ind_nalags ind_na+grocer_nlags(j)+i]
      end
      ind_nalags=unique(ind_nalags)
      matj(ind_nalags,:)=[]
      grocer_mats(j)=matj
   end
 
elseif isempty(grocer_b0) & ~grocer_mindat then
   grocer_delnonna=[2 grocer_nonna(2:$)-grocer_nonna(1:$-1) 2]
   grocer_breaks=find(grocer_delnonna ~= 1)
   grocer_delbreaks=grocer_breaks(2:$)-grocer_breaks(1:$-1)
   [grocer_leng,grocer_pos]=max(grocer_delbreaks)
   grocer_indmat=grocer_nonna(grocer_breaks(grocer_pos)+[0:grocer_leng-1])
   if size(grocer_indmat,2) < size(grocer_nonna,2) then
      disp('warning: your data contain multiple non na periods: the largest non na period has been chosen')
   end

   if grocer_prests then
      grocer_b=[num2date(dmin+grocer_indmat(1)-1,fq) ; num2date(dmin+grocer_indmat($)-1,fq)]
   end

   for j=1:grocer_n
      matj=grocer_mats(j)
      grocer_mats(j)=matj([grocer_indmat(1)+grocer_lags(j):grocer_indmat(1)-1 grocer_indmat grocer_indmat($)+1:grocer_indmat($)+grocer_lags(j)],:)
   end
 

       
elseif grocer_testna then
 
   for j=1:grocer_n
      presna=find(or(isnan(grocer_mats(j)),'r'))
      namesj=grocer_names(j)
      for i=1:size(presna,2)
          error('series '+namesj(presna(i))+' contains Nan in the range you have specified')
      end
   end
 
end
 
endfunction
 
