function [y,namey,grocer_prests,grocer_b,grocer_nonna]=explone(grocer_ly,grocer_b,grocer_named,grocer_testna,grocer_dropna,grocer_mindat,grocer_lags)
 
// PURPOSE: explode several variables into matrices, names and
// define corresponding bounds
// ------------------------------------------------------------
// INPUT:
// * grocer_ly = list of variables:
//   each element could be
//   - a timeseries, a real vector,
//   a real matrix or a string (the name of a variable with
//   one of the types cited above, between quotes)
//   - a matrix of strings, each one being the name of a
//   variable
//   - the string 'cte' or 'const' if the user wants a constant
//     to be included automatically
// * grocer_b = a (px1) string vector (of dates) (optional:
//   if not given the function either takes the existing bounds
//   or determines the bounds suitable to the given series)
// * grocer_named = a (2x1) vector of strings representing the
//   name of variable not entered between quotes
//   (optional; default = ['endogenous' ; 'exogenous'])
// * grocer_testna = a boolean indicating whether the program
//   will test the existence of na’s values in the matrices of
//   values y
// * grocer_dropna = a boolean indicating whether the program
//   should keep only lines of matrices y and grocer_x
//   with non na values in both matrices (suppose that
//   grocer_testna has been set to %f)
// * grocer_mindat =
//   - %t if the user wants to take the min and max dates
//   over which any series exists
//   - %f if the user wants to take the time period over which
//   all series exist
// * grocer_lags = a (n x 1) vector, each coordinate indicating
//   the lag needed for the corresponding elements in the list
//   grocer_vars
// ------------------------------------------------------------
// OUTPUT:
// * y = a (T x ky) real vector or a ts
// * namey = a (ky x 1) string vector
// * prests = a boolean indicating whether there is a ts
// * grocer_b = a (2 x 1) string matrix (of dates) or []
// * grocer_nonna = a (T x 1) vector indicating the indexes of
//   non NA observations
// ------------------------------------------------------------
// Copyright: Eric Dubois 2004-2015
// http://dubois.ensae.net/grocer.html
 
 
[grocer_nargout,grocer_nargin] = argn(0)
if grocer_nargin < 7 then
   grocer_lags=0
end
if grocer_nargin < 6 then
   grocer_mindat=%f
end
if grocer_nargin < 5 then
   grocer_dropna=%f
end
if grocer_nargin < 4 then
   grocer_testna=%t
end
if grocer_nargin < 3 then
   grocer_named='variable'
end
if grocer_nargin < 2 then
   grocer_b = []
end
grocer_prests=%f
 
if ~isempty(grocer_b) then
// bounds are entered in the function: force the presence of ts
   grocer_prests=%t

elseif isempty(grocer_b) & exists('grocer_boundsvar') then
// bounds have been defined outside the function
// but this can be irrelevant; so do not already conclude to ts
   grocer_b=grocer_boundsvar
end
 
 
// explode the list of variables into its various components (real matrices, ts, tsmat, specific variables)
// and their indexes
[namey,listtsy,vecy,indtsy,indvecy,indctey,ny,listtsmaty,indtsmaty]=...
   explovars(grocer_ly,grocer_named)

if ~isempty(indtsy) then
   ts1=listtsy(1)
   fq=ts1('freq')
elseif ~isempty(indtsmaty) then
   tsmat1=listtsmaty(1)
   fq=tsmat1('freq')
end

grocer_prests = (length(listtsy) ~= 0 | length(listtsmaty) ~= 0) | grocer_prests
isvec = (size(vecy,1) ~= 0)
grocer_b0=grocer_b
 
if grocer_prests then
   if isvec then
      warning('your regression contains ts as well as vectors')
   end
 
   if grocer_b == [] then
      dates_start=[]
      dates_end=[]
      if ~isempty(listtsy) then
         [datei_start,datei_end]=ts_span(listtsy)
         dates_start=[dates_start ; datei_start+grocer_lags]
         dates_end=[dates_end ; datei_end]
      end
      if ~isempty(listtsmaty) then
         [datei_start,datei_end]=ts_span(listtsmaty)
         dates_start=[dates_start ; datei_start+grocer_lags]
         dates_end=[dates_end ; datei_end]
      end
      if ~grocer_testna & grocer_mindat then
         dmin=min(dates_start)
         dmax=max(dates_end)
      else
         dmin=max(dates_start)
         dmax=min(dates_end)         
      end
      boundsnum=[dmin:dmax]'
      grocer_b=[num2date(dmin,fq) ; num2date(dmax,fq)]

   else
      boundsnum=[date2num(grocer_b(1)):date2num(grocer_b(2))]'
      for i=2:size(grocer_b,1)/2
         boundsnum=[boundsnum ; [date2num(grocer_b(2*i-1)):date2num(grocer_b(2*i))]']
      end

   end
   nobs=size(boundsnum,1) 

   nmaty=size(vecy,1) // number of observations stemming from the real matrices
   if nmaty > 0 & nmaty ~= nobs then
      error('nobs of matrices do not match with those of ts')
   end
 
   y=%nan*ones(nobs+grocer_lags,ny) // y = vector of values to fill
   b1=[boundsnum(1)-[grocer_lags:-1:0]' ;boundsnum(2:$)]
   for k=1:size(indtsy,1)
      tsk=listtsy(k)
      tsk_dates=tsk('dates')
      start_y=max(1,tsk_dates(1)-b1(1)+1)
      start_tsk=max(1,b1(1)-tsk_dates(1)+1)
      end_y=nobs+grocer_lags-max(0,b1($)-tsk_dates($))
      end_tsk=size(tsk_dates,1)-max(0,tsk_dates($)-b1($))
      y(start_y:end_y,indtsy(k))=tsk('series')(start_tsk:end_tsk,:) // fill the values stemming from ts
   end

   ind0=0
   for k=1:length(listtsmaty)
      tsmatk=listtsmaty(k)
      tsmatk_dates=tsmatk('dates')
      ind_dates=b1-tsmatk_dates(1)+1
      n_dates=size(tsmatk_dates,1)
      ind_dates=ind_dates(ind_dates <= n_dates)
      n_dates=size(ind_dates,1)
      ind1=size(tsmatk('series'),2)
      y(ind_dates(1)+[0:n_dates-1],indtsmaty(ind0+[1:ind1]))=tsmatk('series')(ind_dates,:) // fill the values stemming from ts
      ind0=ind1
   end
     
else
// there is no ts: only real matrices and specifc objects
   if isempty(indvecy) & ~isempty(indctey) then
      error('list of variables cannot contain only specific values, such as const, trend^p')
   end
 
   nobs=size(vecy,1)
   y=ones(nobs,ny)
 
end
 
y(:,indvecy)=vecy // fill the concerned yj columns with their values
 
for i=1:size(indctey,1)
   k=indctey(i)
   y=deal_varspec(y,namey(k),k,boundsnum) // fill specific variables
end
 
grocer_nonna=find(and(~isnan(y),'c'))
 
if grocer_dropna then
   y=y(grocer_nonna,:)
 
elseif isempty(grocer_b0) & ~grocer_mindat then
   grocer_delnonna=[2 grocer_nonna(2:$)-grocer_nonna(1:$-1) 2]
   grocer_breaks=find(grocer_delnonna ~= 1)
   grocer_delbreaks=grocer_breaks(2:$)-grocer_breaks(1:$-1)
   [grocer_leng,grocer_pos]=max(grocer_delbreaks)
   grocer_indmat=grocer_nonna(grocer_breaks(grocer_pos)+[0:grocer_leng-1])
   if size(grocer_indmat,2) < size(grocer_nonna,2) then
      disp('warning: your data contain multiple non na periods: the largest non na period has been chosen')
   end
   if grocer_prests then
      grocer_b=[num2date(dmin+grocer_indmat(1)-1,fq) ; num2date(dmin+grocer_indmat($)-1,fq)]
   end

   y=y(grocer_indmat,:)

elseif grocer_testna then
 
   presna=find(or(isnan(y),'r'))
   for i=1:size(presna,2)
       error('series '+namey(presna(i))+' contains Nan in the range you have specified')
   end
 
end
 
endfunction
 
